@@ -29,7 +29,6 @@ import { CommandRegistry } from '@lumino/commands';
 import { OutputArea } from '@jupyterlab/outputarea';
 import { KernelMessage } from '@jupyterlab/services';
 import { IHeader, IStreamMsg } from '@jupyterlab/services/lib/kernel/messages';
-import { IKernelConnection } from '@jupyterlab/services/lib/kernel/kernel';
 import { JudgeModel } from '../model';
 import { ProblemProvider } from '../problemProvider/problemProvider';
 import { ToolbarItems } from '../toolbar';
@@ -39,6 +38,7 @@ import { BoxPanel, SplitPanel, Widget } from '@lumino/widgets';
 import { JudgeTerminal } from './JudgeTerminal';
 import { JudgeSubmissionArea } from './JudgeSubmissionArea';
 import { SubmissionList } from '../components/SubmissionList';
+import { IKernelConnection } from '@jupyterlab/services/lib/kernel/kernel';
 
 interface IRunResult {
   status: 'OK' | 'TLE' | 'OLE' | 'RE';
@@ -46,6 +46,13 @@ interface IRunResult {
   cpuTime: number;
 }
 
+export class JudgeError extends Error {
+  constructor(message?: string) {
+    super(message);
+    this.name = this.constructor.name;
+  }
+}
+
 export namespace JudgePanel {
   export interface IOptions {
     editorConfig: Partial<CodeEditor.IConfig>;
@@ -283,6 +290,7 @@ export class JudgePanel extends BoxPanel {
   // This is called by command
   public async judge(): Promise<void> {
     const problem = this.model.problem;
+    const code = this.model.source;
 
     if (problem === null) {
       throw new Error('Problem cannot be found.');
@@ -332,9 +340,47 @@ export class JudgePanel extends BoxPanel {
       totalCount: testCases.length
     };
 
+    const waitIdleState = new Promise<void>((resolve, reject) => {
+      const resolveOnIdleState = (
+        sender: ISessionContext,
+        state: KernelMessage.Status
+      ) => {
+        if (state === 'idle') {
+          sessionContext.statusChanged.disconnect(resolveOnIdleState);
+          resolve();
+        }
+      };
+      if (sessionContext.kernelDisplayStatus === 'idle') {
+        resolve();
+      } else {
+        sessionContext.statusChanged.connect(resolveOnIdleState);
+      }
+    });
+
+    // Wait up to 5000 ms for the status of the kernel.
+    const KERNEL_TIMEOUT_MS = 5000;
+    let timer: number | undefined;
+    await Promise.race([
+      waitIdleState,
+      new Promise((resolve, reject) => {
+        timer = setTimeout(
+          () =>
+            reject(
+              new JudgeError(
+                this._trans.__('Kernel is not responding. Please try again.')
+              )
+            ),
+          KERNEL_TIMEOUT_MS
+        );
+      })
+    ]);
+    if (timer) {
+      clearTimeout(timer);
+    }
+
     const results: IRunResult[] = [];
     for (const testCase of testCases) {
-      const result = await this.runWithInput(kernel, problem, testCase);
+      const result = await this.runWithInput(kernel, code, problem, testCase);
       results.push(result);
       this.model.submissionStatus = {
         type: 'progress',
@@ -365,7 +411,7 @@ export class JudgePanel extends BoxPanel {
     const submission = await this.model.submit({
       problemId: problem.id,
       status: status,
-      code: this.model.source,
+      code,
       cpuTime:
         results.map(result => result.cpuTime).reduce((a, b) => a + b, 0) /
         results.length,
@@ -387,41 +433,16 @@ export class JudgePanel extends BoxPanel {
 
   private async runWithInput(
     kernel: IKernelConnection,
+    code: string,
     problem: ProblemProvider.IProblem,
-    input: string,
-    restartKernel = false
+    input: string
   ): Promise<IRunResult> {
-    const code = this.model.source;
-
     const content: KernelMessage.IExecuteRequestMsg['content'] = {
       code,
       stop_on_error: true,
       allow_stdin: true
     };
 
-    if (restartKernel) {
-      await kernel.restart();
-    }
-
-    // TODO 최대 대기 시간 정의
-    const waitIdleState = new Promise<void>((resolve, reject) => {
-      const resolveOnIdleState = (
-        sender: IKernelConnection,
-        state: KernelMessage.Status
-      ) => {
-        if (state === 'idle') {
-          kernel.statusChanged.disconnect(resolveOnIdleState);
-          resolve();
-        }
-      };
-      if (kernel.status === 'idle') {
-        resolve();
-      } else {
-        kernel.statusChanged.connect(resolveOnIdleState);
-      }
-    });
-    await waitIdleState;
-
     let inputLinesLeft: string[] = [];
     if (problem.inputTransferType === 'one_line') {
       inputLinesLeft = input.split(/\r?\n/);
